@@ -12,6 +12,8 @@ from pathlib import Path
 from typing import Any
 
 import typer
+from beartype import beartype
+from icontract import ensure, require
 from rich.console import Console
 from rich.progress import Progress, SpinnerColumn, TextColumn
 
@@ -23,120 +25,35 @@ app = typer.Typer(help="Synchronize Spec-Kit artifacts and repository changes")
 console = Console()
 
 
-def _sync_speckit_to_specfact(repo: Path, converter: Any, scanner: Any, progress: Any) -> tuple[PlanBundle, int, int]:
-    """
-    Sync Spec-Kit artifacts to SpecFact format.
-
-    Returns:
-        Tuple of (merged_bundle, features_updated, features_added)
-    """
-    from specfact_cli.generators.plan_generator import PlanGenerator
-    from specfact_cli.utils.structure import SpecFactStructure
-    from specfact_cli.validators.schema import validate_plan_bundle
-
-    plan_path = repo / SpecFactStructure.DEFAULT_PLAN
-    existing_bundle: PlanBundle | None = None
-
-    if plan_path.exists():
-        validation_result = validate_plan_bundle(plan_path)
-        if isinstance(validation_result, tuple):
-            is_valid, _error, bundle = validation_result
-            if is_valid and bundle:
-                existing_bundle = bundle
-
-    # Convert Spec-Kit to SpecFact
-    converted_bundle = converter.convert_plan(None if not existing_bundle else plan_path)
-
-    # Merge with existing plan if it exists
-    features_updated = 0
-    features_added = 0
-
-    if existing_bundle:
-        feature_keys_existing = {f.key for f in existing_bundle.features}
-
-        for feature in converted_bundle.features:
-            if feature.key in feature_keys_existing:
-                existing_idx = next(i for i, f in enumerate(existing_bundle.features) if f.key == feature.key)
-                existing_bundle.features[existing_idx] = feature
-                features_updated += 1
-            else:
-                existing_bundle.features.append(feature)
-                features_added += 1
-
-        # Update product themes
-        themes_existing = set(existing_bundle.product.themes)
-        themes_new = set(converted_bundle.product.themes)
-        existing_bundle.product.themes = list(themes_existing | themes_new)
-
-        # Write merged bundle
-        generator = PlanGenerator()
-        generator.generate(existing_bundle, plan_path)
-        return existing_bundle, features_updated, features_added
-    # Write new bundle
-    generator = PlanGenerator()
-    generator.generate(converted_bundle, plan_path)
-    return converted_bundle, 0, len(converted_bundle.features)
-
-
-@app.command("spec-kit")
-def sync_spec_kit(
-    repo: Path = typer.Option(
-        Path("."),
-        "--repo",
-        help="Path to repository",
-        exists=True,
-        file_okay=False,
-        dir_okay=True,
-    ),
-    bidirectional: bool = typer.Option(
-        False,
-        "--bidirectional",
-        help="Enable bidirectional sync (Spec-Kit ↔ SpecFact)",
-    ),
-    plan: Path | None = typer.Option(
-        None,
-        "--plan",
-        help="Path to SpecFact plan bundle for SpecFact → Spec-Kit conversion (default: .specfact/plans/main.bundle.yaml)",
-    ),
-    overwrite: bool = typer.Option(
-        False,
-        "--overwrite",
-        help="Overwrite existing Spec-Kit artifacts (delete all existing before sync)",
-    ),
-    watch: bool = typer.Option(
-        False,
-        "--watch",
-        help="Watch mode for continuous sync",
-    ),
-    interval: int = typer.Option(
-        5,
-        "--interval",
-        help="Watch interval in seconds (default: 5)",
-        min=1,
-    ),
+@beartype
+@require(lambda repo: repo.exists(), "Repository path must exist")
+@require(lambda repo: repo.is_dir(), "Repository path must be a directory")
+@require(lambda bidirectional: isinstance(bidirectional, bool), "Bidirectional must be bool")
+@require(lambda plan: plan is None or isinstance(plan, Path), "Plan must be None or Path")
+@require(lambda overwrite: isinstance(overwrite, bool), "Overwrite must be bool")
+@ensure(lambda result: result is None, "Must return None")
+def _perform_sync_operation(
+    repo: Path,
+    bidirectional: bool,
+    plan: Path | None,
+    overwrite: bool,
 ) -> None:
     """
-    Sync changes between Spec-Kit artifacts and SpecFact.
+    Perform sync operation without watch mode.
 
-    Synchronizes markdown artifacts generated by Spec-Kit slash commands
-    with SpecFact plan bundles and protocols.
+    This is extracted to avoid recursion when called from watch mode callback.
 
-    Example:
-        specfact sync spec-kit --repo . --bidirectional
+    Args:
+        repo: Path to repository
+        bidirectional: Enable bidirectional sync
+        plan: Path to SpecFact plan bundle
+        overwrite: Overwrite existing Spec-Kit artifacts
     """
     from specfact_cli.importers.speckit_converter import SpecKitConverter
     from specfact_cli.importers.speckit_scanner import SpecKitScanner
     from specfact_cli.utils.structure import SpecFactStructure
     from specfact_cli.validators.schema import validate_plan_bundle
 
-    console.print(f"[bold cyan]Syncing Spec-Kit artifacts from:[/bold cyan] {repo}")
-
-    # Watch mode (not implemented yet)
-    if watch:
-        console.print("[yellow]→ Watch mode enabled (not implemented yet)[/yellow]")
-        console.print(f"[dim]Would watch for changes every {interval} seconds[/dim]")
-        raise typer.Exit(0)
-
     # Step 1: Detect Spec-Kit repository
     scanner = SpecKitScanner(repo)
     if not scanner.is_speckit_repo():
@@ -317,6 +234,175 @@ def sync_spec_kit(
     console.print("[bold green]✓[/bold green] Sync complete!")
 
 
+def _sync_speckit_to_specfact(repo: Path, converter: Any, scanner: Any, progress: Any) -> tuple[PlanBundle, int, int]:
+    """
+    Sync Spec-Kit artifacts to SpecFact format.
+
+    Returns:
+        Tuple of (merged_bundle, features_updated, features_added)
+    """
+    from specfact_cli.generators.plan_generator import PlanGenerator
+    from specfact_cli.utils.structure import SpecFactStructure
+    from specfact_cli.validators.schema import validate_plan_bundle
+
+    plan_path = repo / SpecFactStructure.DEFAULT_PLAN
+    existing_bundle: PlanBundle | None = None
+
+    if plan_path.exists():
+        validation_result = validate_plan_bundle(plan_path)
+        if isinstance(validation_result, tuple):
+            is_valid, _error, bundle = validation_result
+            if is_valid and bundle:
+                existing_bundle = bundle
+
+    # Convert Spec-Kit to SpecFact
+    converted_bundle = converter.convert_plan(None if not existing_bundle else plan_path)
+
+    # Merge with existing plan if it exists
+    features_updated = 0
+    features_added = 0
+
+    if existing_bundle:
+        feature_keys_existing = {f.key for f in existing_bundle.features}
+
+        for feature in converted_bundle.features:
+            if feature.key in feature_keys_existing:
+                existing_idx = next(i for i, f in enumerate(existing_bundle.features) if f.key == feature.key)
+                existing_bundle.features[existing_idx] = feature
+                features_updated += 1
+            else:
+                existing_bundle.features.append(feature)
+                features_added += 1
+
+        # Update product themes
+        themes_existing = set(existing_bundle.product.themes)
+        themes_new = set(converted_bundle.product.themes)
+        existing_bundle.product.themes = list(themes_existing | themes_new)
+
+        # Write merged bundle
+        generator = PlanGenerator()
+        generator.generate(existing_bundle, plan_path)
+        return existing_bundle, features_updated, features_added
+    # Write new bundle
+    generator = PlanGenerator()
+    generator.generate(converted_bundle, plan_path)
+    return converted_bundle, 0, len(converted_bundle.features)
+
+
+@app.command("spec-kit")
+def sync_spec_kit(
+    repo: Path = typer.Option(
+        Path("."),
+        "--repo",
+        help="Path to repository",
+        exists=True,
+        file_okay=False,
+        dir_okay=True,
+    ),
+    bidirectional: bool = typer.Option(
+        False,
+        "--bidirectional",
+        help="Enable bidirectional sync (Spec-Kit ↔ SpecFact)",
+    ),
+    plan: Path | None = typer.Option(
+        None,
+        "--plan",
+        help="Path to SpecFact plan bundle for SpecFact → Spec-Kit conversion (default: .specfact/plans/main.bundle.yaml)",
+    ),
+    overwrite: bool = typer.Option(
+        False,
+        "--overwrite",
+        help="Overwrite existing Spec-Kit artifacts (delete all existing before sync)",
+    ),
+    watch: bool = typer.Option(
+        False,
+        "--watch",
+        help="Watch mode for continuous sync",
+    ),
+    interval: int = typer.Option(
+        5,
+        "--interval",
+        help="Watch interval in seconds (default: 5)",
+        min=1,
+    ),
+) -> None:
+    """
+    Sync changes between Spec-Kit artifacts and SpecFact.
+
+    Synchronizes markdown artifacts generated by Spec-Kit slash commands
+    with SpecFact plan bundles and protocols.
+
+    Example:
+        specfact sync spec-kit --repo . --bidirectional
+    """
+
+    console.print(f"[bold cyan]Syncing Spec-Kit artifacts from:[/bold cyan] {repo}")
+
+    # Resolve repo path to ensure it's absolute and valid (do this once at the start)
+    resolved_repo = repo.resolve()
+    if not resolved_repo.exists():
+        console.print(f"[red]Error:[/red] Repository path does not exist: {resolved_repo}")
+        raise typer.Exit(1)
+    if not resolved_repo.is_dir():
+        console.print(f"[red]Error:[/red] Repository path is not a directory: {resolved_repo}")
+        raise typer.Exit(1)
+
+    # Watch mode implementation
+    if watch:
+        from specfact_cli.sync.watcher import FileChange, SyncWatcher
+
+        console.print("[bold cyan]Watch mode enabled[/bold cyan]")
+        console.print(f"[dim]Watching for changes every {interval} seconds[/dim]\n")
+
+        @beartype
+        @require(lambda changes: isinstance(changes, list), "Changes must be a list")
+        @require(
+            lambda changes: all(hasattr(c, "change_type") for c in changes),
+            "All changes must have change_type attribute",
+        )
+        @ensure(lambda result: result is None, "Must return None")
+        def sync_callback(changes: list[FileChange]) -> None:
+            """Handle file changes and trigger sync."""
+            spec_kit_changes = [c for c in changes if c.change_type == "spec_kit"]
+            specfact_changes = [c for c in changes if c.change_type == "specfact"]
+
+            if spec_kit_changes or specfact_changes:
+                console.print(f"[cyan]Detected {len(changes)} change(s), syncing...[/cyan]")
+                # Perform one-time sync (bidirectional if enabled)
+                try:
+                    # Re-validate resolved_repo before use (may have been cleaned up)
+                    if not resolved_repo.exists():
+                        console.print(f"[yellow]⚠[/yellow] Repository path no longer exists: {resolved_repo}\n")
+                        return
+                    if not resolved_repo.is_dir():
+                        console.print(f"[yellow]⚠[/yellow] Repository path is no longer a directory: {resolved_repo}\n")
+                        return
+                    # Use resolved_repo from outer scope (already resolved and validated)
+                    _perform_sync_operation(
+                        repo=resolved_repo,
+                        bidirectional=bidirectional,
+                        plan=plan,
+                        overwrite=overwrite,
+                    )
+                    console.print("[green]✓[/green] Sync complete\n")
+                except Exception as e:
+                    console.print(f"[red]✗[/red] Sync failed: {e}\n")
+
+        # Use resolved_repo for watcher (already resolved and validated)
+        watcher = SyncWatcher(resolved_repo, sync_callback, interval=interval)
+        watcher.watch()
+        return
+
+    # Perform sync operation (extracted to avoid recursion in watch mode)
+    # Use resolved_repo (already resolved and validated above)
+    _perform_sync_operation(
+        repo=resolved_repo,
+        bidirectional=bidirectional,
+        plan=plan,
+        overwrite=overwrite,
+    )
+
+
 @app.command("repository")
 def sync_repository(
     repo: Path = typer.Option(
@@ -364,16 +450,65 @@ def sync_repository(
 
     console.print(f"[bold cyan]Syncing repository changes from:[/bold cyan] {repo}")
 
-    if target is None:
-        target = repo / ".specfact"
+    # Resolve repo path to ensure it's absolute and valid (do this once at the start)
+    resolved_repo = repo.resolve()
+    if not resolved_repo.exists():
+        console.print(f"[red]Error:[/red] Repository path does not exist: {resolved_repo}")
+        raise typer.Exit(1)
+    if not resolved_repo.is_dir():
+        console.print(f"[red]Error:[/red] Repository path is not a directory: {resolved_repo}")
+        raise typer.Exit(1)
 
-    sync = RepositorySync(repo, target, confidence_threshold=confidence)
+    if target is None:
+        target = resolved_repo / ".specfact"
+
+    sync = RepositorySync(resolved_repo, target, confidence_threshold=confidence)
 
     if watch:
-        console.print("[yellow]→ Watch mode enabled (not implemented yet)[/yellow]")
-        console.print(f"[dim]Would watch for changes every {interval} seconds[/dim]")
-        raise typer.Exit(0)
+        from specfact_cli.sync.watcher import FileChange, SyncWatcher
 
+        console.print("[bold cyan]Watch mode enabled[/bold cyan]")
+        console.print(f"[dim]Watching for changes every {interval} seconds[/dim]\n")
+
+        @beartype
+        @require(lambda changes: isinstance(changes, list), "Changes must be a list")
+        @require(
+            lambda changes: all(hasattr(c, "change_type") for c in changes),
+            "All changes must have change_type attribute",
+        )
+        @ensure(lambda result: result is None, "Must return None")
+        def sync_callback(changes: list[FileChange]) -> None:
+            """Handle file changes and trigger sync."""
+            code_changes = [c for c in changes if c.change_type == "code"]
+
+            if code_changes:
+                console.print(f"[cyan]Detected {len(code_changes)} code change(s), syncing...[/cyan]")
+                # Perform repository sync
+                try:
+                    # Re-validate resolved_repo before use (may have been cleaned up)
+                    if not resolved_repo.exists():
+                        console.print(f"[yellow]⚠[/yellow] Repository path no longer exists: {resolved_repo}\n")
+                        return
+                    if not resolved_repo.is_dir():
+                        console.print(f"[yellow]⚠[/yellow] Repository path is no longer a directory: {resolved_repo}\n")
+                        return
+                    # Use resolved_repo from outer scope (already resolved and validated)
+                    result = sync.sync_repository_changes(resolved_repo)
+                    if result.status == "success":
+                        console.print("[green]✓[/green] Repository sync complete\n")
+                    elif result.status == "deviation_detected":
+                        console.print(f"[yellow]⚠[/yellow] Deviations detected: {len(result.deviations)}\n")
+                    else:
+                        console.print(f"[red]✗[/red] Sync failed: {result.status}\n")
+                except Exception as e:
+                    console.print(f"[red]✗[/red] Sync failed: {e}\n")
+
+        # Use resolved_repo for watcher (already resolved and validated)
+        watcher = SyncWatcher(resolved_repo, sync_callback, interval=interval)
+        watcher.watch()
+        return
+
+    # Use resolved_repo (already resolved and validated above)
     with Progress(
         SpinnerColumn(),
         TextColumn("[progress.description]{task.description}"),
@@ -381,7 +516,7 @@ def sync_repository(
     ) as progress:
         # Step 1: Detect code changes
         task = progress.add_task("Detecting code changes...", total=None)
-        result = sync.sync_repository_changes(repo)
+        result = sync.sync_repository_changes(resolved_repo)
         progress.update(task, description=f"✓ Detected {len(result.code_changes)} code changes")
 
         # Step 2: Show plan updates
